package cbg.xdoc;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.Signature;

import com.thoughtworks.qdox.JavaDocBuilder;
import com.thoughtworks.qdox.model.DocletTag;
import com.thoughtworks.qdox.model.JavaClass;
import com.thoughtworks.qdox.model.JavaField;
import com.thoughtworks.qdox.model.JavaMethod;
public class Categories {
	private void addAccessors(JavaClass clazz, JavaMethod method) {
		String name = method.getName();
		if (!name.startsWith("get") && !name.startsWith("set"))
			return;
		if (name.length() == 3)
			return;
		int index = name.indexOf("get");
		if (index == -1)
			index = name.indexOf("set");
		index += 3;
		name = name.substring(index);
		if (classHasField(clazz, name)) {
			DocletTag[] tagArray = method.getTags();
			List tags = new ArrayList(tagArray.length + 1);
			tags.add(new DocletTag("cat", "accessing"));
			method.setTags(tags);
		}
	}

	private void addConstructor(JavaMethod method) {
		if(!method.isConstructor()) return;
		DocletTag[] tagArray = method.getTags();
		List tags = new ArrayList(tagArray.length + 1);
		tags.add(new DocletTag("cat", "constructing"));
		method.setTags(tags);
	}
	private boolean classHasField(JavaClass clazz, String name) {
		// hack
		JavaField[] fields = clazz.getFields();
		for (int i = 0; i < fields.length; i++) {
			JavaField field = fields[i];
			/* Probably should do a real case check here. */
			if (field.getName().equalsIgnoreCase(name))
				return true;
		}
		return false;
	}
	private void fillMethodMap(IType type) {
		try {
			JavaDocBuilder builder = new JavaDocBuilder();
			builder.addSource(new StringReader(type.getSource()));
			JavaClass clazz = builder.getSources()[0].getClasses()[0];
			IMethod[] methods = type.getMethods();
			int j = 0;
			int methodCount = clazz.getMethods().length;
			for (int i = 0; j < methodCount; i++, j++) {
				JavaMethod method = clazz.getMethods()[i];
				IMethod eMethod = methods[j];
				if (eMethod.isConstructor() && !method.isConstructor()) {
					i--;
					continue;
				}
				addAccessors(clazz, method);
				addConstructor(method);
				addTags(method.getTags());
				addMethodToMap(method, eMethod);
			}
		} catch(JavaModelException jme) {
		    //ignore
		}
	}
	private void addMethodToMap(JavaMethod qdoxMethod, IMethod eMethod) throws JavaModelException {
		String[] parameters = eMethod.getParameterTypes();
		String returnType = eMethod.getReturnType();
		String sig = Signature.createMethodSignature(parameters, returnType);
		char[] convertedSig = new char[sig.length()];
		for (int k = 0; k < convertedSig.length; k++) {
			char c = sig.charAt(k);
			convertedSig[k] = (c == '.' ? '/' : c);
		}
		methodMap.put(qdoxMethod.getName() + new String(convertedSig), qdoxMethod);
	}
	private void addTags(DocletTag[] tags) {
		for (int i = 0; i < tags.length; i++) {
			addTag(tags[i]);
		}
	}
	private void addTag(DocletTag docletTag) {
		ValueSet values = (ValueSet) allTags.get(docletTag.getName());
		if (values == null) {
			values = new ValueSet(docletTag.getName());
			allTags.put(docletTag.getName(), values);
		}
		values.addValue(docletTag.getValue());
	}
	protected Map methodMap = new HashMap();
	protected Map allTags;
	public static final DocletTag[] EMPTY = new DocletTag[0];
	public static final String TAG_CATEGORY = "category";
	public static final String TAG_CAT = "cat";
	public Categories() {
		super();
		allTags = new TreeMap();
	}
	public boolean hasCategoryTag(IMethod method) {
		return hasTag(method, TAG_CAT);
	}
	public DocletTag[] getTags(IMethod method) {
		try {
			String unique = method.getElementName() + method.getSignature();
			JavaMethod qdoxMethod = (JavaMethod) methodMap.get(unique);
			if(qdoxMethod == null) return EMPTY;
			DocletTag[] tags = qdoxMethod.getTags();
			return (tags == null ? EMPTY : tags); 
		} catch (JavaModelException e) {
		    //ignore
		}
		return EMPTY;
	}
	public boolean hasTag(IMethod method, String tag) {
		DocletTag[] tags = getTags(method);
		for (int i = 0; i < tags.length; i++) {
			DocletTag aTag = tags[i];
			if (aTag.getName().equals(tag))
				return true;
		}
		return false;
	}
	/**
	 * Answer a map of all the Javadoc tags found for a class.
	 * The map has this shape:
	 * tag "foo" -> TagSet
	 * tag "stu" -> TagSet 
	 * @return Map
	 */
	public Map getAllTags() {
		return allTags;
	}
	public static Categories newCategory(IType type) {
		Categories category = new Categories();
		category.parseSource(type);
		return category;
	}
	protected void parseSource(IType type) {
		if (type != null)
			fillMethodMap(type);
	}
	public boolean hasTagWithValue(IMethod method, ValueSet.Value value) {
		return hasTagWithValue(method, value.getParent().getName(), value.getValue());
	}
	public boolean hasTagWithValue(IMethod method, String tagName, String value) {
		DocletTag[] tags = getTags(method);
		for (int i = 0; i < tags.length; i++) {
			DocletTag aTag = tags[i];
			if (aTag.getName().equals(tagName) && aTag.getValue().equals(value))
				return true;
		}
		return false;
	}
	public void reparse(IType type) {
		allTags.clear();
		parseSource(type);
	}
}
