package cbg.xdoc;

import java.util.Set;
import java.util.TreeSet;

/**
 * The TagSet class contains the values for a paticular tag.
 * so 
 * *cat dog
 * *cat foo
 * *cat stu
 * If *cat was the Javadoc tag then aTagSet would contain. dog, foo, stu
 * @author chrisg
 *
 */
public class ValueSet {
	protected Set values;
	protected String name;
	
	public class Value implements Comparable {
		protected ValueSet parent;
		protected String value;
		
		public Value(ValueSet parent, String value) {
			this.parent = parent;
			this.value = value;
		}
		public ValueSet getParent() {
			return parent;
		}

		public String toString() {
			return value;
		}

		public int compareTo(Object o) {
			return value.compareTo(((ValueSet.Value)o).value);
		}

		public String getValue() {
			return value;
		}

	}
	public ValueSet(String name) {
		super();
		this.name = name;
		values = new TreeSet();
	}
	public Set getValues() {
		return values;
	}
	public void addValue(String value) {
		values.add(new Value(this, value));
	}
	public String getName() {
		return name;
	}

	public boolean equals(Object obj) {
		if (!(obj instanceof ValueSet)) return false;
		return name.equals(((ValueSet)obj).name);
	}

	public int hashCode() {
		return name.hashCode();
	}

}
