package cbg.xdoc.views;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jdt.core.ElementChangedEvent;
import org.eclipse.jdt.core.IClassFile;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IElementChangedListener;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaElementDelta;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.ui.JavaPlugin;
import org.eclipse.jdt.internal.ui.browsing.MembersView;
import org.eclipse.jdt.internal.ui.browsing.PackagesView;
import org.eclipse.jdt.internal.ui.browsing.TypesView;
import org.eclipse.jdt.internal.ui.javaeditor.IClassFileEditorInput;
import org.eclipse.jdt.ui.IWorkingCopyManager;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchPartSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.DrillDownAdapter;
import org.eclipse.ui.part.ViewPart;
import cbg.xdoc.AllCategories;
import cbg.xdoc.Categories;
import cbg.xdoc.CategoryFilter;
import cbg.xdoc.ValueSet;
import cbg.xdoc.XdocPlugin;

public class CategoryView extends ViewPart {
    protected TreeViewer viewer;
    private DrillDown drillDownAdapter;
    protected Action doubleClickAction;
    protected IType type;
    protected Object previousSelection;
    public static final Object[] EMPTY = new Object[0];
    protected CategoryFilter categoryFilter;
    public final String MEMBERS_VIEW = "org.eclipse.jdt.ui.MembersView";
    protected StructuredViewer membersViewer;
    private IPartListener partListener;
    private boolean processSelectionEvents = true;
    private IElementChangedListener elementChangedListener;
    private ISelectionListener partSelectionListener;
    private IWorkbenchPart previousPart;
    private ISelectionChangedListener viewerListener;
    private Action deselectAction;
    private IMemento fMemento;

    class DrillDown extends DrillDownAdapter {
        protected Object topInto;
        protected Object[] topExpanded;

        public DrillDown(TreeViewer tree) {
            super(tree);
        }

        public void goHome() {
            super.goHome();
            clear();
        }

        public void goInto(Object newInput) {
            // This assumes you never descend more than one level
            topInto = newInput;
            topExpanded = viewer.getExpandedElements();
            super.goInto(newInput);
        }

        public void goIntoSimilar(Object newInput) {
            // This assumes you never descend more than one level
            topInto = newInput;
            topExpanded = viewer.getExpandedElements();
            Widget similarInput = viewer.testFindItem(newInput);
            /*
             * Apparently the viewer's new input does not contain what we're
             * trying to descend into, so just return.
             */
            if (similarInput == null) return;
            super.goInto(similarInput.getData());
        }

        public void reset() {
            super.reset();
            clear();
        }

        protected void clear() {
            topInto = null;
            topExpanded = null;
        }

        public void goBack() {
            super.goBack();
            clear();
        }
    }
    class TagLabelProvider extends LabelProvider {
        public String getText(Object element) {
            if (element instanceof ValueSet) { return ((ValueSet) element).getName(); }
            return super.getText(element);
        }
    }
    class CategoryContentProvider implements ITreeContentProvider {
        public Object[] getChildren(Object parentElement) {
            if (parentElement instanceof Categories) {
                Categories cats = (Categories) parentElement;
                Map tagMap = cats.getAllTags();
                Collection tags = tagMap.values();
                return (ValueSet[]) tags.toArray(new ValueSet[tags.size()]);
            } else if (parentElement instanceof ValueSet) {
                Set tags = ((ValueSet) parentElement).getValues();
                return (ValueSet.Value[]) tags.toArray(new ValueSet.Value[tags.size()]);
            }
            return EMPTY;
        }

        public Object getParent(Object element) {
            if (element instanceof ValueSet.Value) return ((ValueSet.Value) element).getParent();
            return null;
        }

        public boolean hasChildren(Object element) {
            return getChildren(element).length > 0;
        }

        public Object[] getElements(Object inputElement) {
            return getChildren(inputElement);
        }

        public void dispose() {
            //ignore
        }

        public void inputChanged(Viewer aViewer, Object oldInput, Object newInput) {
            //ignore
        }
    }

    /**
     * The constructor.
     */
    public CategoryView() {
        //ignore
    }

    /**
     * This is a callback that will allow us to create the viewer and
     * initialize it.
     */
    public void createPartControl(Composite parent) {
        viewer = new TreeViewer(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
        viewer.setUseHashlookup(true);
        drillDownAdapter = new DrillDown(viewer);
        viewer.setContentProvider(new CategoryContentProvider());
        viewer.setLabelProvider(new TagLabelProvider());
        //viewer.setInput(ResourcesPlugin.getWorkspace());
        makeActions();
        hookContextMenu();
        hookDoubleClickAction();
        contributeToActionBars();
        hookListeners();
        //getSite().setSelectionProvider(viewer);
    }

    private void postRefresh() {
        postRunnable(new Runnable() {
            public void run() {
                Control ctrl = viewer.getControl();
                if (ctrl != null && !ctrl.isDisposed()) {
                    ctrl.setRedraw(false);
                    viewer.refresh(viewer.getInput());
                    viewer.setSelection(viewer.getSelection());
                    ctrl.setRedraw(true);
                }
            }
        });
    }

    private void postRunnable(final Runnable r) {
        Control ctrl = viewer.getControl();
        if (ctrl != null && !ctrl.isDisposed()) {
            Display currentDisplay = Display.getCurrent();
            if (currentDisplay != null && currentDisplay.equals(ctrl.getDisplay())) ctrl.getDisplay().syncExec(r);
            else
                ctrl.getDisplay().asyncExec(r);
        }
    }

    /**
     * Processes a delta recursively. When more than two children are affected
     * the tree is fully refreshed starting at this node. The delta is
     * processed in the current thread but the viewer updates are posted to the
     * UI thread.
     */
    protected void processDelta(IJavaElementDelta delta) throws JavaModelException {
        int kind = delta.getKind();
        int flags = delta.getFlags();
        IJavaElement element = delta.getElement();
        if (!getProvideWorkingCopy() && element instanceof ICompilationUnit && ((ICompilationUnit) element).isWorkingCopy()) return;
        if (element != null && element.getElementType() == IJavaElement.COMPILATION_UNIT && !isOnClassPath((ICompilationUnit) element)) return;
        // handle open and closing of a solution or project
        if (((flags & IJavaElementDelta.F_CLOSED) != 0) || ((flags & IJavaElementDelta.F_OPENED) != 0)) {
            postRefresh();
            return;
        }
        if (kind == IJavaElementDelta.REMOVED) {
            //ignore
        }
        if (kind == IJavaElementDelta.ADDED && delta.getMovedFromElement() != null && element instanceof ICompilationUnit) return;
        if (kind == IJavaElementDelta.ADDED) {
            //ignore
        }
        if (kind == IJavaElementDelta.CHANGED) {
            //ignore
        }
        //		if (isClassPathChange(delta))
        //			 // throw the towel and do a full refresh
        //			postRefresh(null);
        //		if ((flags & IJavaElementDelta.F_ARCHIVE_CONTENT_CHANGED) != 0 &&
        // fInput instanceof IJavaElement) {
        //			IPackageFragmentRoot pkgRoot= (IPackageFragmentRoot)element;
        //			IJavaElement inputsParent=
        // ((IJavaElement)fInput).getAncestor(IJavaElement.PACKAGE_FRAGMENT_ROOT);
        //			if (pkgRoot.equals(inputsParent))
        //				postRefresh(null);
        //		}
        //
        //		// the source attachment of a JAR has changed
        //		if (element instanceof IPackageFragmentRoot && (((flags &
        // IJavaElementDelta.F_SOURCEATTACHED) != 0 || ((flags &
        // IJavaElementDelta.F_SOURCEDETACHED)) != 0)))
        //			postUpdateIcon(element);
        if (element instanceof IType) {
            reparseType((IType) element, true);
            return;
        }
        if (element instanceof ICompilationUnit && (flags & IJavaElementDelta.F_CONTENT) != 0) {
            reparseClass((ICompilationUnit) element);
            return;
        }
        IJavaElementDelta[] affectedChildren = delta.getAffectedChildren();
        if (affectedChildren.length > 1) {
            //ignore
        }
        for (int i = 0; i < affectedChildren.length; i++) {
            processDelta(affectedChildren[i]);
        }
    }

    private void reparseClass(ICompilationUnit iCompilationUnit) {
        IType[] types = null;
        try {
            types = iCompilationUnit.getAllTypes();
        } catch (JavaModelException e) {
            return;
        }
        for (int i = 0; i < types.length; i++) {
            reparseType(types[i], false);
        }
        viewer.refresh(viewer.getInput());
        viewer.setSelection(viewer.getSelection());
    }

    private void reparseType(IType aType, boolean updateViewer) {
        AllCategories.reparse(aType);
        System.out.println("Reparse " + aType.getFullyQualifiedName());
        if (updateViewer) {
            postRefresh();
        }
    }

    private boolean getProvideWorkingCopy() {
        return true;
    }

    private boolean isOnClassPath(ICompilationUnit element) {
        IJavaProject project = element.getJavaProject();
        if (project == null || !project.exists()) return false;
        return project.isOnClasspath(element);
    }

    private void hookListeners() {
        elementChangedListener = new IElementChangedListener() {
            public void elementChanged(ElementChangedEvent event) {
                IJavaElementDelta delta = event.getDelta();
                try {
                    processDelta(delta);
                } catch (JavaModelException e) {
                    //ignore
                }
            }
        };
        JavaCore.addElementChangedListener(elementChangedListener);
        partSelectionListener = new ISelectionListener() {
            public void selectionChanged(IWorkbenchPart part, ISelection selection) {
                doPartSelectionChanged(part, selection);
            }
        };
        getSite().getWorkbenchWindow().getSelectionService().addSelectionListener(partSelectionListener);
        viewerListener = new ISelectionChangedListener() {
            public void selectionChanged(SelectionChangedEvent event) {
                doCategorySelection(event);
            }
        };
        viewer.addSelectionChangedListener(viewerListener);
        // Add a listener for the text editor
        partListener = new IPartListener() {
            public void partActivated(IWorkbenchPart part) {
                if (part instanceof IEditorPart) {
                    doSelectionFromEditor(part);
                }
            }

            public void partBroughtToTop(IWorkbenchPart part) {
                if (part instanceof IEditorPart) {
                    doSelectionFromEditor(part);
                }
            }

            public void partClosed(IWorkbenchPart part) {
                if (part instanceof MembersView) {
                    membersViewer = null;
                    categoryFilter = null;
                }
            }

            public void partDeactivated(IWorkbenchPart part) {
                //ignore
            }

            public void partOpened(IWorkbenchPart part) {
                if (part instanceof MembersView) {
                    if (membersViewer == null) {
                        getCategoryFilter();
                    }
                }
            }
        };
        getViewSite().getPage().addPartListener(partListener);
    }

    protected void doCategorySelection(SelectionChangedEvent event) {
        CategoryFilter filter = getCategoryFilter();
        if (filter == null) return;
        filter.showCategories(event.getSelection());
        if (membersViewer == null) return;
        membersViewer.setInput(membersViewer.getInput());
    }

    public void dispose() {
        if (viewer != null) {
            getViewSite().getWorkbenchWindow().getSelectionService().removeSelectionListener(partSelectionListener);
            getViewSite().getPage().removePartListener(partListener);
            JavaCore.removeElementChangedListener(elementChangedListener);
            viewer = null;
        }
        super.dispose();
    }

    protected void doPartSelectionChanged(IWorkbenchPart part, ISelection iSelection) {
        if (!processSelectionEvents || !(iSelection instanceof IStructuredSelection)) return;
        if (!(part instanceof TypesView) && !(part instanceof PackagesView)) return;
        Object selection = getSingleElementFromSelection(iSelection);
        if (selection != null && selection.equals(previousSelection)) return;
        System.out.println("do p: " + part.getClass());
        previousSelection = selection;
        previousPart = part;
        if (part instanceof TypesView && selection != type) {
            adjustInputAndSetSelection((IJavaElement) selection);
        } else if (part instanceof PackagesView && type != null && selection != null) {
            if (type.getPackageFragment().equals(selection)) {
                System.out.println("\t!!same package");
                return;
            }
            adjustInputAndSetSelection(null);
        }
        System.out.println("\t!!bail");
    }

    protected void doSelectionFromEditor(IWorkbenchPart part) {
        if (!XDocPreferencePage.linkBrowsingViewSelectionToEditor()) return;
        if (part == null || previousPart == part) {
            System.out.println("\t# bail");
            return;
        }
        previousPart = part;
        IWorkbenchPartSite site = part.getSite();
        if (site == null) return;
        ISelectionProvider provider = site.getSelectionProvider();
        if (provider != null) setSelectionFromEditor(part, provider.getSelection());
    }

    private void setSelectionFromEditor(IWorkbenchPart part, ISelection selection) {
        if (part instanceof IEditorPart) {
            IJavaElement element = null;
            if (selection instanceof IStructuredSelection) {
                Object obj = getSingleElementFromSelection(selection);
                if (obj instanceof IJavaElement) element = (IJavaElement) obj;
            }
            IEditorInput ei = ((IEditorPart) part).getEditorInput();
            if (selection instanceof ITextSelection) {
                int offset = ((ITextSelection) selection).getOffset();
                element = getElementForInputAt(ei, offset);
            }
            if (element != null) {
                adjustInputAndSetSelection(element);
                return;
            }
            if (ei instanceof IFileEditorInput) {
                IFile file = ((IFileEditorInput) ei).getFile();
                IJavaElement je = (IJavaElement) file.getAdapter(IJavaElement.class);
                if (je == null) {
                    IContainer container = ((IFileEditorInput) ei).getFile().getParent();
                    if (container != null) je = (IJavaElement) container.getAdapter(IJavaElement.class);
                }
                if (je == null) {
                    setSelection(null, false);
                    return;
                }
                adjustInputAndSetSelection(je);
            } else if (ei instanceof IClassFileEditorInput) {
                IClassFile cf = ((IClassFileEditorInput) ei).getClassFile();
                adjustInputAndSetSelection(cf);
            }
        }
    }

    protected void adjustInputAndSetSelection(IJavaElement je) {
        // Remember expanded items
        Object[] expanded = viewer.getExpandedElements();
        processSelectionEvents = false;
        viewer.getTree().setRedraw(false);
        Object previousTop = drillDownAdapter.topInto;
        Object[] previousExpanded = drillDownAdapter.topExpanded;
        drillDownAdapter.reset();
        System.out.println("adjust input");
        try {
            while (je != null) {
                if (je instanceof IType) {
                    updateInput(je);
                    return;
                }
                je = je.getParent();
            }
            updateInput(null);
        } finally {
            processSelectionEvents = true;
            if (previousTop != null) {
                viewer.setExpandedElements(previousExpanded);
                drillDownAdapter.goIntoSimilar(previousTop);
            } else {
                viewer.setExpandedElements(expanded);
            }
            viewer.getTree().setRedraw(true);
        }
    }

    protected void setInitialSelection() {
        // Use the selection, if any
        Object input;
        IWorkbenchPage page = getSite().getPage();
        ISelection selection = null;
        if (page != null) selection = page.getSelection();
        if (selection instanceof ITextSelection) {
            Object part = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActivePart();
            if (part instanceof IEditorPart) {
                doSelectionFromEditor((IEditorPart) part);
                if (viewer.getSelection() != null) return;
            }
        }
        if (selection != null && !selection.isEmpty()) input = getSingleElementFromSelection(selection);
        else {
            // Use the input of the page
            input = getSite().getPage().getInput();
            if (!(input instanceof IJavaElement)) {
                if (input instanceof IAdaptable) input = ((IAdaptable) input).getAdapter(IJavaElement.class);
                else
                    return;
            }
        }
        if (input instanceof IType) {
            adjustInputAndSetSelection((IJavaElement) input);
        }
    }

    protected void restoreState(IMemento memento) {
        viewer.getControl().setRedraw(false);
        viewer.refresh();
        viewer.getControl().setRedraw(true);
    }

    /*
     * Implements method from IViewPart.
     */
    public void saveState(IMemento memento) {
        if (viewer == null) {
            // part has not been created
            if (fMemento != null) //Keep the old state;
                    memento.putMemento(fMemento);
            return;
        }
    }

    /*
     * Implements method from IViewPart.
     */
    public void init(IViewSite site, IMemento memento) throws PartInitException {
        super.init(site, memento);
        fMemento = memento;
    }

    protected void setSelection(ISelection selection, boolean reveal) {
        if (selection != null && selection.equals(viewer.getSelection())) return;
        processSelectionEvents = false;
        viewer.setSelection(selection, reveal);
        processSelectionEvents = true;
    }

    /**
     * @see JavaEditor#getElementAt(int)
     */
    protected IJavaElement getElementForInputAt(IEditorInput input, int offset) {
        if (input instanceof IClassFileEditorInput) {
            try {
                IJavaElement element = ((IClassFileEditorInput) input).getClassFile().getElementAt(offset);
                if (element == null) { return ((IClassFileEditorInput) input).getClassFile().getType(); }
                return element;
            } catch (JavaModelException ex) {
                return null;
            }
        }
        IWorkingCopyManager manager = JavaPlugin.getDefault().getWorkingCopyManager();
        ICompilationUnit unit = manager.getWorkingCopy(input);
        if (unit != null) {
            try {
                unit.reconcile(false, null);
                return unit.getElementAt(offset);
            } catch (JavaModelException ex) {
                //ignore
            }
        }
        return null;
    }

    protected final Object getSingleElementFromSelection(ISelection selection) {
        if (!(selection instanceof StructuredSelection) || selection.isEmpty()) return null;
        Iterator iter = ((StructuredSelection) selection).iterator();
        Object firstElement = iter.next();
        if (!(firstElement instanceof IJavaElement)) {
            if (firstElement instanceof IAdaptable) {
                IJavaElement je = (IJavaElement) ((IAdaptable) firstElement).getAdapter(IJavaElement.class);
                if (je == null && firstElement instanceof IFile) {
                    IContainer parent = ((IFile) firstElement).getParent();
                    if (parent != null) return (IJavaElement) parent.getAdapter(IJavaElement.class);
                    else
                        return null;
                } else
                    return je;
            } else
                return firstElement;
        }
        Object currentInput = this.type;
        if (currentInput == null || !currentInput.equals(findInputForJavaElement((IJavaElement) firstElement))) if (iter.hasNext()) // multi
                // selection
                // and
                // view
                // is
                // empty
                return null;
                else
                    // ok: single selection and view is empty
                    return firstElement;
        // be nice to multi selection
        while (iter.hasNext()) {
            Object element = iter.next();
            if (!(element instanceof IJavaElement)) return null;
            if (!currentInput.equals(findInputForJavaElement((IJavaElement) element))) return null;
        }
        return firstElement;
    }

    /**
     * Finds the closest Java element which can be used as input for this part
     * and has the given Java element as child
     * 
     * @param je
     *            the Java element for which to search the closest input
     * @return the closest Java element used as input for this part
     */
    protected IJavaElement findInputForJavaElement(IJavaElement je) {
        if (je == null || !je.exists()) return null;
        if (isValidInput(je)) return je;
        return findInputForJavaElement(je.getParent());
    }

    protected boolean isValidInput(IJavaElement je) {
        return je instanceof IType;
    }

    protected CategoryFilter getCategoryFilter() {
        if (categoryFilter == null) {
            /*
             * Attempt to find the categoryFilter. This is painful, but I
             * couldn't find a better way....
             */
            MembersView members = getMembersView();
            if (members == null) return null;
            try {
                Method method = members.getClass().getSuperclass().getDeclaredMethod("getViewer", null);
                method.setAccessible(true);
                //Method method = members.getClass().getMethod("getViewer",
                // null);
                membersViewer = (StructuredViewer) method.invoke(members, null);
            } catch (NoSuchMethodException e) {
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            } catch (InvocationTargetException e) {
                e.printStackTrace();
            }
            ViewerFilter[] filters = membersViewer.getFilters();
            for (int i = 0; i < filters.length; i++) {
                ViewerFilter filter = filters[i];
                if (filter instanceof CategoryFilter) {
                    categoryFilter = (CategoryFilter) filter;
                    return categoryFilter;
                }
            }
        }
        return categoryFilter;
    }

    protected MembersView getMembersView() {
        IViewPart view = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().findView(MEMBERS_VIEW);
        return (MembersView) view;
    }

    private void updateInput(Object input) {
        System.out.println("*update tree " + input);
        type = null;
        if (input == null) {
            // clear input
            viewer.setInput(null);
            return;
        }
        if (input instanceof IType) {
            IType aType = (IType) input;
            this.type = aType;
            try {
                if (aType.getSource() == null) return;
            } catch (JavaModelException e) {
                return;
            }
            viewer.setInput(AllCategories.getCategory(aType));
        }
    }

    private void hookContextMenu() {
        MenuManager menuMgr = new MenuManager("#PopupMenu");
        menuMgr.setRemoveAllWhenShown(true);
        menuMgr.addMenuListener(new IMenuListener() {
            public void menuAboutToShow(IMenuManager manager) {
                CategoryView.this.fillContextMenu(manager);
            }
        });
        Menu menu = menuMgr.createContextMenu(viewer.getControl());
        viewer.getControl().setMenu(menu);
        getSite().registerContextMenu(menuMgr, viewer);
    }

    private void contributeToActionBars() {
        IActionBars bars = getViewSite().getActionBars();
        fillLocalPullDown(bars.getMenuManager());
        fillLocalToolBar(bars.getToolBarManager());
    }

    private void fillLocalPullDown(IMenuManager manager) {
        manager.add(deselectAction);
        //		manager.add(new Separator());
        //		manager.add(action2);
    }

    protected void fillContextMenu(IMenuManager manager) {
        //		manager.add(action1);
        //		manager.add(action2);
        //		manager.add(new Separator());
        drillDownAdapter.addNavigationActions(manager);
        //		// Other plug-ins can contribute there actions here
        //		manager.add(new Separator("Additions"));
    }

    private void fillLocalToolBar(IToolBarManager manager) {
        drillDownAdapter.addNavigationActions(manager);
        manager.add(new Separator());
        manager.add(deselectAction);
    }

    private void makeActions() {
        deselectAction = new Action() {
            public void run() {
                if (viewer != null) viewer.setSelection(new StructuredSelection());
            }
        };
        deselectAction.setText("Deselect All");
        deselectAction.setToolTipText("Deselect All");
        deselectAction.setImageDescriptor(XdocPlugin.getImageDescriptor("collapseall.gif"));
    }

    private void hookDoubleClickAction() {
        viewer.addDoubleClickListener(new IDoubleClickListener() {
            public void doubleClick(DoubleClickEvent event) {
                //				doubleClickAction.run();
            }
        });
    }

    /**
     * Passing the focus request to the viewer's control.
     */
    public void setFocus() {
        viewer.getControl().setFocus();
    }
}